# ----------------------------------------------------------------------------------------------------------------------
# anecdotica.py - Python-библиотека для работы с API сайта anecdotica.ru
# Python 3.8
# ver. 1.2, 03.09.2022, Андрей Шевченко <anecdotika@yandex.ru>
# Изменения:
# + параметр country для API Lev. 1 (getRandItem)
# + API Lev. 3 (getRandItemT)
# доработан метод send_query в AnecdoticaApiCore
# ----------------------------------------------------------------------------------------------------------------------
import json
import hashlib
import logging
import requests
import time
from urllib.parse import quote_plus

# ----------------------------------------------------------------------------------------------------------------------
# Общие настройки
# ----------------------------------------------------------------------------------------------------------------------
API_URL = 'http://anecdotica.ru/api'  # http-адрес API
# Настройки для режима отладки
API_LOGFILE = 'api_errors.log'  # путь к файлу лога
API_LOGGING = 0  # логирование (0 - выкл., 1 - вкл.)

if API_LOGGING:
    logging.basicConfig(filename=API_LOGFILE, level=logging.INFO, format='%(asctime)s : %(levelname)s : %(message)s',
                        datefmt='%Y-%m-%d %H:%M:%S')  # настройка формата вывода в лог
# ----------------------------------------------------------------------------------------------------------------------


class AnecdoticaApiCore:
    """Базовый класс"""
    DEFAULT_SETTINGS = {'charset': 'utf-8', 'format': 'json'}
    reply = None
    called_from = None

    def __init__(self, profile, settings=None, handler=None):
        self.profile = profile
        self.settings = settings
        self.handler = handler

    def set_param(self, name, value):
        self.settings[name] = value

    @staticmethod
    def log_error(result):
        """Запись информации об ошибке в лог"""
        if API_LOGGING and result:  # если логирование включено
            msg = str(result.get_error()) + ' (' + result.get_err_msg() + ')'
            logging.error(msg)  # пишем в лог

    @staticmethod
    def set_error(err_code, err_msg):
        """Возвращаем информацию об ошибке"""
        return json.dumps({'result': {'error': err_code, 'errMsg': err_msg}})

    @staticmethod
    def get_setting(settings, name):
        """Получение значения параметра с именем name"""
        return settings.get(name)

    @staticmethod
    def send_query(params, profile):
        """Выполнение запроса к API"""
        content = None
        params['uts'] = int(time.time())

        try:
            url_params = ''
            for key in params:
                if url_params == '':
                    und = ''
                else:
                    und = '&'
                if params[key] is not None:
                    if isinstance(params[key], str):
                        params[key] = quote_plus(params[key])
                    url_params = url_params + und + key + '=' + str(params[key])

            md5hash = hashlib.md5((url_params + profile['key']).encode())
            api_url = API_URL + '?' + url_params + '&hash=' + md5hash.hexdigest()

            if profile['http_method'] == 'GET':
                content = requests.get(api_url)  # GET-запрос
            elif profile['http_method'] == 'POST':
                content = requests.post(api_url)  # POST-запрос
            else:
                content = AnecdoticaApiCore.set_error(7, 'FORBIDDEN_HTTP_METHOD')

            if content is None:
                content = AnecdoticaApiCore.set_error(103, 'NO_CONTENT_RECEIVED')
            elif hasattr(content, 'status_code'):
                if content.status_code == 200:  # 200
                    content = content.text
                else:  # http-error
                    content = AnecdoticaApiCore.set_error(102, 'HTTP_REQUEST_FAILED')

        except (ConnectionError, ConnectionRefusedError):
            content = AnecdoticaApiCore.set_error(102, 'HTTP_REQUEST_FAILED')
        except Exception as e:
            content = AnecdoticaApiCore.set_error(102, 'HTTP_REQUEST_FAILED')

        reply = json.loads(content)
        return reply

    def get_next_reply(self, handler=None):
        self.reply = self._get_reply(self.called_from, self.profile, self.settings,
                                     self.handler if handler is None else handler)
        return self.reply

    @staticmethod
    def _get_reply(cls, profile, settings=None, handler=None):
        params = cls.set_params(profile, settings)
        reply = ApiReply(cls.send_query(params, profile))
        if reply.is_error():
            AnecdoticaApiCore.log_error(reply.get_result())
            if handler is not None:
                handler.on_error(reply.get_result())
        else:
            if handler is not None:
                handler.on_success(reply.get_data())
        return reply


class ApiReply:
    """Класс ответа сервера"""
    item = None
    items = None

    def __init__(self, data):
        self.result = Result(data.get('result'))
        if data.get('item') is not None:
            self.item = Item(data.get('item'))
        if data.get('items') is not None:
            self.items = Items(data.get('items'), data.get('info'))

    def is_error(self):
        return self.result.is_error()

    def get_result(self):
        return self.result

    def get_item(self):
        return self.item

    def get_items(self):
        return self.items

    def get_data(self):
        data = None
        if self.item is not None:
            data = self.item
        elif self.items is not None:
            data = self.items
        return data


class DataElement:
    """Класс элемента данных"""
    data = {}

    def __init__(self, data):
        self.data = data

    def as_array(self):
        return self.data


class Result(DataElement):
    """Класс элемента данных Result"""

    def get_error(self):
        return None if self.data['error'] == '' else self.data['error']

    def get_err_msg(self):
        return None if self.data['errMsg'] == '' else self.data['errMsg']

    def is_error(self):
        return not (self.data['error'] == '' or self.data['error'] == 0)


class Item(DataElement):
    """Класс записи Item"""

    def get_text(self):
        text = self.data.get('text')
        if text is None:
            text = ''
        return text

    def get_note(self):
        text = self.data.get('note')
        if text is None:
            text = ''
        return text


class Items(DataElement):
    """Класс списка записей Items"""
    info = None
    index = 0

    def __init__(self, items, info):
        super().__init__(items)
        self.info = info

    def reset(self):
        self.index = 0

    def get_item(self, index=None):
        if index is None:
            index = self.index
            self.index += 1
        return Item(self.data[index])

    def has_item(self, index=None):
        if index is None:
            index = self.index
        if 0 <= index < len(self.data):
            result = self.data[index] is not None
        else:
            result = False
        return result

    def get_num(self):
        return 0 if self.info.get('n') is None else self.info.get('n')

    def get_size(self):
        return 0 if self.info.get('size') is None else self.info.get('size')

    def get_page(self):
        return 0 if self.info.get('page') is None else self.info.get('page')

    def get_max_page(self):
        return 0 if self.info.get('max_page') is None else self.info.get('max_page')

    def get_ipp(self):
        return 0 if self.info.get('ipp') is None else self.info.get('ipp')


class RandomItemApi(AnecdoticaApiCore):
    """API Level 1"""
    DEF_METHOD = 'getRandItem'

    def __init__(self, profile, settings=None, handler=None):
        super().__init__(profile, settings, handler)
        self.called_from = __class__

    @staticmethod
    def get_reply(profile, settings=None, handler=None):
        reply = AnecdoticaApiCore._get_reply(__class__, profile, settings, handler)
        return reply

    @staticmethod
    def set_params(profile, settings):
        return {
            'pid': profile['pid'],
            'method': __class__.DEF_METHOD,
            'format': __class__.DEFAULT_SETTINGS['format'],
            'charset': __class__.DEFAULT_SETTINGS['charset'],
            'category': __class__.get_setting(settings, 'category'),
            'genre': __class__.get_setting(settings, 'genre'),
            'country': __class__.get_setting(settings, 'country'),
            'lang': __class__.get_setting(settings, 'lang'),
            'markup': __class__.get_setting(settings, 'markup'),
            'note': __class__.get_setting(settings, 'note')
        }


class RandomItemParamApi(AnecdoticaApiCore):
    """API Level 2"""
    DEF_METHOD = 'getRandItemP'

    def __init__(self, profile, settings=None, handler=None):
        super().__init__(profile, settings, handler)
        self.called_from = __class__

    @staticmethod
    def get_reply(profile, settings=None, handler=None):
        reply = AnecdoticaApiCore._get_reply(__class__, profile, settings, handler)
        return reply

    @staticmethod
    def set_params(profile, settings):
        return {
            'pid': profile['pid'],
            'method': __class__.DEF_METHOD,
            'format': __class__.DEFAULT_SETTINGS['format'],
            'charset': __class__.DEFAULT_SETTINGS['charset'],
            'tag': __class__.get_setting(settings, 'tag'),
            'category': __class__.get_setting(settings, 'category'),
            'series': __class__.get_setting(settings, 'series'),
            'country': __class__.get_setting(settings, 'country'),
            'genre': __class__.get_setting(settings, 'genre'),
            'lang': __class__.get_setting(settings, 'lang'),
            'wlist': __class__.get_setting(settings, 'wlist'),
            'censor': __class__.get_setting(settings, 'censor'),
            'markup': __class__.get_setting(settings, 'markup'),
            'note': __class__.get_setting(settings, 'note')
        }


class RandomItemTagsApi(AnecdoticaApiCore):
    """API Level 3"""
    DEF_METHOD = 'getRandItemT'

    def __init__(self, profile, settings=None, handler=None):
        super().__init__(profile, settings, handler)
        self.called_from = __class__

    @staticmethod
    def get_reply(profile, settings=None, handler=None):
        reply = AnecdoticaApiCore._get_reply(__class__, profile, settings, handler)
        return reply

    @staticmethod
    def set_params(profile, settings):
        return {
            'pid': profile['pid'],
            'method': __class__.DEF_METHOD,
            'format': __class__.DEFAULT_SETTINGS['format'],
            'charset': __class__.DEFAULT_SETTINGS['charset'],
            'tags': __class__.get_setting(settings, 'tags'),
            'precision': __class__.get_setting(settings, 'precision'),
            'priority': __class__.get_setting(settings, 'priority'),
            'category': __class__.get_setting(settings, 'category'),
            'series': __class__.get_setting(settings, 'series'),
            'country': __class__.get_setting(settings, 'country'),
            'genre': __class__.get_setting(settings, 'genre'),
            'lang': __class__.get_setting(settings, 'lang'),
            'wlist': __class__.get_setting(settings, 'wlist'),
            'censor': __class__.get_setting(settings, 'censor'),
            'markup': __class__.get_setting(settings, 'markup'),
            'note': __class__.get_setting(settings, 'note')
        }


class ItemsApi(AnecdoticaApiCore):
    """API Level 4"""
    DEF_METHOD = 'getItems'

    def __init__(self, profile, settings=None, handler=None):
        super().__init__(profile, settings, handler)
        self.called_from = __class__

    def get_page(self, page=0, handler=None):
        self.settings['page'] = page
        self.reply = self._get_reply(self.called_from, self.profile, self.settings,
                                     self.handler if handler is None else handler)
        return self.reply

    @staticmethod
    def get_reply(profile, settings=None, handler=None):
        reply = AnecdoticaApiCore._get_reply(__class__, profile, settings, handler)
        return reply

    @staticmethod
    def set_params(profile, settings):
        return {
            'pid': profile['pid'],
            'method': __class__.DEF_METHOD,
            'format': __class__.DEFAULT_SETTINGS['format'],
            'charset': __class__.DEFAULT_SETTINGS['charset'],
            'page': __class__.get_setting(settings, 'page'),
            'ipp': __class__.get_setting(settings, 'ipp'),
            'tag': __class__.get_setting(settings, 'tag'),
            'category': __class__.get_setting(settings, 'category'),
            'series': __class__.get_setting(settings, 'series'),
            'country': __class__.get_setting(settings, 'country'),
            'genre': __class__.get_setting(settings, 'genre'),
            'lang': __class__.get_setting(settings, 'lang'),
            'wlist': __class__.get_setting(settings, 'wlist'),
            'censor': __class__.get_setting(settings, 'censor'),
            'markup': __class__.get_setting(settings, 'markup'),
            'note': __class__.get_setting(settings, 'note')
        }


class ApiHandlerInterface:
    """Класс обработки результатов запроса"""

    # Замечание по использованию собственных обработчиков:
    # 1) создать класс, наследующий ApiHandlerInterface
    # 2) переопределить методы класса on_success и on_error.

    def on_success(self, data):  # data: Item | Items
        """Пользовательский обработчик при успешном запросе"""
        # Вызывается при отсутствии ошибки
        pass

    def on_error(self, error: Result):
        """Пользовательский обработчик ошибки"""
        # Вызывается в случае ошибки (код и описание ошибки)
        pass



import ipih

from DataSourceService.const import *
from pih.collections import (
    EventDS,
    GKeepItem,
    DelayedMessage,
    DelayedMessageDS,
    MessageSearchCritery,
    PolibasePersonVisitDS,
    CTIndicationsValueContainer,
    PolibasePersonInformationQuest,
    PolibasePersonVisitNotification,
    ChillerIndicationsValueContainer,
    PolibasePersonVisitNotificationDS,
    PolibasePersonNotificationConfirmation,
)
from pih.consts.errors import Error
from pih.consts import MessageStatuses
from pih import A, PIHThreadPoolExecutor
from pih.consts.names import KEYWORD_COLLECTION
from pih.tools import if_else, j, js, escs, ne, e, n, nn, one


import json
import dataclasses
from lmdbm import Lmdb
from typing import Any, Callable
from datetime import datetime, date
from mysql.connector import pooling
from mysql.connector import ProgrammingError
from concurrent.futures import Future, as_completed
from mysql.connector.pooling import PooledMySQLConnection

executor: PIHThreadPoolExecutor = PIHThreadPoolExecutor(max_workers=1)
executor_task_list: list[Future] = []

class JsonLmdb(Lmdb):
    def _pre_key(self, value):
        return value.encode("utf-8")

    def _post_key(self, value):
        return value.decode("utf-8")

    def _pre_value(self, value):
        return json.dumps(value).encode("utf-8")

    def _post_value(self, value):
        return json.loads(value.decode("utf-8"))

class DataStorageAndSourceApi:
    def __init__(self) -> None:
        self.connection_pool = pooling.MySQLConnectionPool(
            pool_name="pool",
            pool_size=DATABASE_POOL_SIZE,
            pool_reset_session=True,
            host=A.CT_H.BACKUP_WORKER.ALIAS,
            user=A.D_V_E.value(A.CT_LNK.DATABASE_ADMINISTRATOR_LOGIN),
            password=A.D_V_E.value(A.CT_LNK.DATABASE_ADMINISTRATOR_PASSWORD),
            database=DATABASE_NAME,
            port=DATABASE_PORT,
        )
        self.cached_ct_indications_value_container_list: list[
            CTIndicationsValueContainer
        ] = []
        self.cached_chiller_indications_value_container_list: list[
            ChillerIndicationsValueContainer
        ] = []
        self.storage_section_map: dict[str, JsonLmdb] = {}

    def create_connection(self) -> PooledMySQLConnection:
        return self.connection_pool.get_connection()

    def execute(
        self,
        function: Callable[[PooledMySQLConnection], Any | None],
        default_value: Any | None = None,
        use_return_of_action: bool = True,
        catch_exception: bool = False,
    ) -> Any | None:
        result: Any | None = default_value
        connection: PooledMySQLConnection | None = None
        try:
            connection = self.create_connection()
            temp_result: Any | None = function(connection)
            if use_return_of_action:
                result = temp_result
        except ProgrammingError as error:
            A.ER.global_except_hook(error)
            if catch_exception:
                raise Error(error._full_msg, None)
        finally:
            if ne(connection):
                connection.close()
        return result

    def get_settings_value(self, key: str, default_value: Any | None = None) -> Any:
        result: dict | None = self.get_storage_value(key, SETTINGS_SECTION)
        if n(result):
            self.set_settings_value(key, default_value)
        return result

    def set_settings_value(self, key: str, value: Any) -> bool:
        self.set_storage_value(key, value, SETTINGS_SECTION)
        return True

    def get_storage_container(self, section: str) -> JsonLmdb:
        if section not in self.storage_section_map:
            self.storage_section_map[section] = JsonLmdb.open(
                A.PTH.join(A.PTH_DS.VALUE, A.PTH.add_extension(section, "db")),
                "c",
            )
        return self.storage_section_map[section]

    def set_storage_value(
        self, key: str, value: dict | None, section: str | None = None
    ) -> bool:
        section = section or STORAGE_SECTION
        db: JsonLmdb = self.get_storage_container(section)
        if e(value):
            del db[key]
        else:
            db[key] = value
        return True

    def get_storage_value(
        self, key: str | None, section: str | None = None
    ) -> dict | None:
        section = section or STORAGE_SECTION
        db: JsonLmdb = self.get_storage_container(section)
        result: dict | None = None
        if e(key):
            result = {}
            for key in db:
                result[key] = db[key]
        else:
            if key in db:
                result = db[key]
        return result

    def get_events_table_field_list(self) -> list[str]:
        return [A.CT_FNC.NAME, A.CT_FNC.PARAMETERS, A.CT_FNC.TIMESTAMP, A.CT_FNC.ID]

    def create_insert_query(
        self,
        table_name: str,
        data: Any,
        field_list: list[str] | None = None,
        replace: bool = False,
    ) -> str:
        field_list = field_list or A.D.fields(data)
        data_item_list: list[Any] = [
            getattr(data, field.name) for field in dataclasses.fields(data)
        ]
        return js(
            (
                ["insert", "replace"][replace],
                "into",
                table_name,
                A.D.list_to_string(field_list, start="(", end=")"),
                "values",
                "(",
                A.D.list_to_string(
                    list(
                        map(
                            lambda data_item: A.D_F.as_string(
                                data_item,
                                True,
                                lambda item: A.D.check_not_none(
                                    item, lambda: item, "NULL"
                                ),
                            ),
                            data_item_list,
                        )
                    )
                ),
                ")",
            )
        )

    def register_event(self, value: EventDS) -> bool:
        executor_task_list.append(
            executor.submit(
                lambda value: self.execute_event_action(ACTIONS.INSERT, value),
                value,
            )
        )
        for result_holder in as_completed(executor_task_list):
            return result_holder.result()

    def get_event(self, value: EventDS) -> list[EventDS]:
        return self.execute_event_action(ACTIONS.SELECT, value)

    def get_event_count(self, value: EventDS) -> list[EventDS]:
        return self.execute_event_action(ACTIONS.COUNT, value)

    def remove_event(self, value: EventDS) -> bool:
        return self.execute_event_action(ACTIONS.DELETE, value)

    def execute_event_action(
        self, value: ACTIONS, event: EventDS
    ) -> list[EventDS] | bool:
        event_name: str | None = event.name
        parameters: dict[str, Any] | None = event.parameters
        result: list[EventDS] | int | bool = if_else(
            value == ACTIONS.SELECT, [], if_else(value == ACTIONS.COUNT, 0, False)
        )

        def action(
            connection: PooledMySQLConnection, result: list[EventDS] | bool
        ) -> list[EventDS] | bool:
            cursor = connection.cursor(dictionary=True)
            query_string: str | None = None
            if value in [ACTIONS.DELETE, ACTIONS.SELECT, ACTIONS.COUNT]:
                parameter_list: list[Any] = A.D.filter(
                    lambda item: nn(item[1]), list((parameters or {}).items())
                )
                condition_statement_list: list[str] = []
                if ne(event_name):
                    condition_statement_list += [
                        j((A.CT_FNC.NAME, "=", escs(event_name)))
                    ]
                if ne(parameter_list):
                    is_sign_symbol: bool = False
                    is_list: bool = False
                    condition_statement_list += [
                        j(
                            (
                                "parameters->",
                                escs(j(("$.", key))),
                                (
                                    j((" ", value[0], " "))
                                    if (
                                        (is_list := isinstance(value, list))
                                        and (
                                            is_sign_symbol := value[0].lower()
                                            in (
                                                SIGN_COLLECTION
                                                + [KEYWORD_COLLECTION.LIKE]
                                            )
                                        )
                                    )
                                    else "="
                                ),
                                A.D_F.as_string(
                                    (
                                        j((ANY_SYMBOL, value[1], ANY_SYMBOL))
                                        if is_sign_symbol
                                        and value[0].lower() == KEYWORD_COLLECTION.LIKE
                                        else value[1] if is_list else value
                                    ),
                                    escaped_string=True,
                                ),
                            )
                        )
                        for key, value in parameter_list
                    ]
                timestamp: datetime | date | str | int | None = event.timestamp
                if ne(timestamp):
                    if isinstance(timestamp, str):
                        timestamp_sign: str | None = None
                        try:
                            timestamp = A.D.datetime_or_date_from_string(timestamp)
                        except ValueError as _:
                            for sign in SIGN_COLLECTION:
                                if timestamp.startswith(sign):
                                    timestamp = A.D.datetime_or_date_from_string(
                                        timestamp[len(sign) :].lstrip()
                                    )
                                    timestamp_sign = sign
                                    break
                        if n(timestamp_sign):
                            if isinstance(timestamp, datetime):
                                timestamp = j(("=", escs(timestamp)))
                            elif isinstance(timestamp, date):
                                timestamp = j(
                                    (
                                        KEYWORD_COLLECTION.LIKE,
                                        escs(j((event.timestamp, ANY_SYMBOL))),
                                    )
                                )
                        else:
                            timestamp = j((timestamp_sign, escs(timestamp)))
                        condition_statement_list.append(j(("timestamp", timestamp)))
                query_string = js(
                    (
                        if_else(
                            value in [ACTIONS.SELECT, ACTIONS.COUNT], "select", "delete"
                        ),
                        if_else(
                            value == ACTIONS.SELECT,
                            A.D.list_to_string(self.get_events_table_field_list()),
                            if_else(value == ACTIONS.COUNT, "count(*)", ""),
                        ),
                        js(
                            (
                                "from",
                                EVENTS_TABLE,
                                (
                                    ""
                                    if e(condition_statement_list)
                                    else js(
                                        ("where", j(condition_statement_list, " and "))
                                    )
                                ),
                                (
                                    js(
                                        (
                                            "order by timestamp",
                                            "asc" if timestamp > 0 else "desc",
                                            "limit",
                                            abs(timestamp),
                                        )
                                    )
                                    if isinstance(timestamp, int)
                                    else None
                                ),
                            )
                        ),
                    )
                )
            if value == ACTIONS.INSERT:
                query_string = self.create_insert_query(
                    EVENTS_TABLE, event, self.get_events_table_field_list()
                )
            cursor.execute(query_string)
            if value == ACTIONS.SELECT:
                for item in cursor:
                    result.append(
                        A.D.fill_data_from_source(
                            EventDS(), {**item}, copy_by_index=True
                        )
                    )
            if value == ACTIONS.COUNT:
                for item in cursor:
                    result = one({**item})
                    break
            if value in [ACTIONS.DELETE, ACTIONS.INSERT]:
                connection.commit()
                result = cursor.rowcount != 0
            return result

        return self.execute(lambda connection: action(connection, result), result)

    def register_polibase_person_information_quest(
        self, value: PolibasePersonInformationQuest
    ) -> bool:
        return self.execute_insert(
            f"insert into {POLIBASE_PERSON_INFORMATION_QUESTS_TABLE_NAME} ({A.CT_FNC.PIN}, {A.CT_FNC.FULL_NAME}, {A.CT_FNC.TELEPHONE_NUMBER}) value ({value.pin}, '{value.FullName}', '{value.telephoneNumber}')"
        )

    def register_ct_indication_value(
        self, value_container: CTIndicationsValueContainer, forced: bool = False
    ) -> bool:
        self.cached_ct_indications_value_container_list.append(value_container)
        if forced or (
            value_container.timestamp.minute
            % self.get_settings_value(
                A.CT_S.CT_INDICATIONS_VALUE_SAVE_PERIOD_IN_MINUTES.name
            )
            == 0
        ):
            return self.execute_insert(
                f"replace into {CT_INDICATIONS_VALUE_TABLE_NAME} ({A.CT_FNC.TIMESTAMP}, {A.CT_FNC.TEMPERATURE}, {A.CT_FNC.HUMIDITY}) value ('{value_container.timestamp}', {value_container.temperature}, {value_container.humidity})"
            )
        return False

    def register_chiller_indication_value(
        self,
        value_container: ChillerIndicationsValueContainer | None,
        forced: bool = False,
    ) -> ChillerIndicationsValueContainer | None:
        if ne(value_container):
            if A.D_C.INDICATIONS.chiller_value_valid(value_container):
                self.cached_chiller_indications_value_container_list.append(
                    value_container
                )
                if forced or (
                    value_container.timestamp.minute
                    % self.get_settings_value(
                        A.CT_S.CHILLER_INDICATIONS_VALUE_SAVE_PERIOD_IN_MINUTES.name
                    )
                    == 0
                ):
                    self.execute_insert(
                        f"replace into {CHILLER_INDICATIONS_VALUE_TABLE_NAME} ({A.CT_FNC.TIMESTAMP}, {A.CT_FNC.TEMPERATURE}, {A.CT_FNC.INDICATORS}) value ('{value_container.timestamp}', {value_container.temperature}, {value_container.indicators})"
                    )
                return value_container
            else:
                return value_container
        return None

    def update_polibase_person_information_quest(
        self,
        value: PolibasePersonInformationQuest,
        search_critery: PolibasePersonInformationQuest,
    ) -> bool:
        result: bool = False

        def action(connection: PooledMySQLConnection) -> bool:
            cursor = connection.cursor()
            set_statement_string: str | None = None
            field_value: Any = None
            field_value_string: str | None = None
            set_statement_part_list: list[str] = []
            for field_name in value.__dataclass_fields__:
                field_value = value.__getattribute__(field_name)
                if nn(field_value):
                    if isinstance(field_value, str):
                        field_value_string = escs(field_value)
                    else:
                        field_value_string = str(field_value)
                    set_statement_part_list.append(
                        f"{field_name} = {field_value_string}"
                    )
            if len(set_statement_part_list) > 0:
                set_statement_string = "set " + A.D.list_to_string(
                    set_statement_part_list, separator=", "
                )
                cursor.execute(
                    f"update {POLIBASE_PERSON_INFORMATION_QUESTS_TABLE_NAME} {set_statement_string} {self.get_condition_string(search_critery)}"
                )
                connection.commit()
                result = cursor.rowcount == 1
            return result

        return self.execute(action, result)

    def execute_query(self, value: str) -> list[dict]:
        result: list[dict] = []

        def action(connection: PooledMySQLConnection) -> list[dict]:
            # connection.autocommit = True
            cursor = connection.cursor(dictionary=True)
            cursor.execute(value)
            try:
                connection.commit()
            except Exception as _:
                pass
            for item in cursor:
                result.append({**item})
            return result

        return self.execute(action, result, catch_exception=True)

    def get_polibase_person_information_quests(
        self, search_critery: PolibasePersonInformationQuest
    ) -> list[PolibasePersonInformationQuest]:
        result: list[PolibasePersonInformationQuest] = []

        def action(
            connection: PooledMySQLConnection,
        ) -> list[PolibasePersonInformationQuest]:
            cursor = connection.cursor(dictionary=True)
            cursor.execute(
                f"select * from {POLIBASE_PERSON_INFORMATION_QUESTS_TABLE_NAME} {self.get_condition_string(search_critery)}"
            )
            for item in cursor:
                result.append(
                    A.D.fill_data_from_source(
                        PolibasePersonInformationQuest(), {**item}
                    )
                )
            return result

        return self.execute(action, result)

    def get_last_ct_indications_value_container_list(
        self, cached: bool = True, count: int = 1
    ) -> list[CTIndicationsValueContainer]:
        result: list[CTIndicationsValueContainer] = []
        if cached:
            cached_result: list[CTIndicationsValueContainer] | None = (
                self.get_last_cached_indications_value_container_list(
                    self.cached_ct_indications_value_container_list, count
                )
            )
            if ne(cached_result):
                result = cached_result
        if e(result):

            def action(
                connection: PooledMySQLConnection,
            ) -> list[CTIndicationsValueContainer]:
                cursor = connection.cursor(dictionary=True)
                cursor.execute(
                    f"select * from {CT_INDICATIONS_VALUE_TABLE_NAME} ORDER BY {A.CT_FNC.TIMESTAMP} DESC LIMIT {count}"
                )
                for item in cursor:
                    result.append(
                        A.D.fill_data_from_source(
                            CTIndicationsValueContainer(), {**item}
                        )
                    )
                cursor.close()
                return result

            return self.execute(action, result)
        return result

    def get_last_chiller_indications_value_container_list(
        self, cached: bool = True, count: int = 1
    ) -> list[ChillerIndicationsValueContainer]:
        result: list[ChillerIndicationsValueContainer] = []
        if cached:
            cached_result: list[ChillerIndicationsValueContainer] = (
                self.get_last_cached_indications_value_container_list(
                    self.cached_chiller_indications_value_container_list, count
                )
            )
            if ne(cached_result):
                result = cached_result
        if e(result):

            def action(
                connection: PooledMySQLConnection,
            ) -> list[ChillerIndicationsValueContainer]:
                cursor = connection.cursor(dictionary=True)
                cursor.execute(
                    f"select * from {CHILLER_INDICATIONS_VALUE_TABLE_NAME} ORDER BY {A.CT_FNC.TIMESTAMP} DESC LIMIT {count}"
                )
                for item in cursor:
                    result.append(
                        A.D.fill_data_from_source(
                            ChillerIndicationsValueContainer(), {**item}
                        )
                    )
                cursor.close()
                return result

            return self.execute(action, result)
        return result

    def get_last_cached_indications_value_container_list(
        self,
        value_list: list[
            CTIndicationsValueContainer | ChillerIndicationsValueContainer
        ],
        count: int,
    ) -> list[CTIndicationsValueContainer | ChillerIndicationsValueContainer]:
        result_list: list[
            CTIndicationsValueContainer | ChillerIndicationsValueContainer
        ] = []
        for value_container in reversed(value_list):
            if isinstance(value_container, CTIndicationsValueContainer):
                result_list.append(value_container)
            if isinstance(value_container, ChillerIndicationsValueContainer):
                result_list.append(value_container)
            if len(result_list) == count:
                break
        return result_list

    def get_search_polibase_person_visit_notification_query(
        self, search_critery: PolibasePersonVisitNotification | None = None
    ) -> str:
        condition_statement_string: str = self.get_condition_string(search_critery)
        return f"select * from {POLIBASE_PERSON_VISITS_TABLE_NAME} inner join {POLIBASE_PERSON_VISIT_NOTIFICATIONS_TABLE_NAME} on {A.CT_FNC.VISIT_ID} = {A.CT_FNC.ID} {condition_statement_string}"

    def get_search_polibase_person_visit_query(
        self, search_critery: PolibasePersonVisitDS | None = None
    ) -> str:
        condition_statement_string: str = self.get_condition_string(search_critery)
        return f"select * from {POLIBASE_PERSON_VISITS_TABLE_NAME} {condition_statement_string}"

    def search_polibase_person_visit_notifications(
        self, search_critery: PolibasePersonVisitNotification | None = None
    ) -> list[PolibasePersonVisitNotification]:
        result: list[PolibasePersonVisitNotification] = []

        def action(
            connection: PooledMySQLConnection,
        ) -> list[PolibasePersonVisitNotification]:
            cursor = connection.cursor(dictionary=True)
            cursor.execute(
                self.get_search_polibase_person_visit_notification_query(search_critery)
            )
            for item in cursor:
                result.append(
                    A.D.fill_data_from_source(
                        PolibasePersonVisitNotification(), {**item}
                    )
                )
            return result

        return self.execute(action, result)

    def search_polibase_person_visit(
        self, search_critery: PolibasePersonVisitDS
    ) -> list[PolibasePersonVisitDS]:
        class DH:
            result: list[PolibasePersonVisitNotification] = []

        def action(connection: PooledMySQLConnection) -> list[PolibasePersonVisitDS]:
            cursor = connection.cursor(dictionary=True)
            cursor.execute(self.get_search_polibase_person_visit_query(search_critery))
            for item in cursor:
                DH.result.append(
                    A.D.fill_data_from_source(PolibasePersonVisitDS(), {**item})
                )
            return DH.result

        return self.execute(action, DH.result)

    def get_gkeep_item_list_by_any(
        self, name: str | None, title: str | None, full_equaliment: bool | None
    ) -> list[GKeepItem]:
        search_critery_name: str | None = None
        search_critery_title: str | None = None
        equaliment_value: str = "" if full_equaliment else ANY_SYMBOL
        if ne(name):
            search_critery_name = j(
                (
                    "" if full_equaliment else KEYWORD_COLLECTION.LIKE,
                    j((equaliment_value, name.lower(), equaliment_value)),
                )
            )
        if ne(title):
            search_critery_title = j(
                (
                    KEYWORD_COLLECTION.LIKE,
                    j((ANY_SYMBOL, title.lower(), ANY_SYMBOL)),
                )
            )

        class DH:
            search_critery: GKeepItem | None = (
                None
                if e(search_critery_name) and e(search_critery_title)
                else GKeepItem(search_critery_name, search_critery_title)
            )
            result: list[GKeepItem] = []

        def action(connection: PooledMySQLConnection) -> None:
            cursor = connection.cursor(dictionary=True)
            cursor.execute(
                js(
                    (
                        "select",
                        "*",
                        "from",
                        GKEEP_MAP_TABLE,
                        (
                            ""
                            if e(DH.search_critery)
                            else self.get_condition_string(DH.search_critery, False)
                        ),
                    )
                )
            )
            for item in cursor:
                DH.result.append(A.D.fill_data_from_source(GKeepItem(), {**item}))

        return self.execute(action, DH.result, False)

    def add_gkeep_item(self, value: GKeepItem) -> bool:
        value.name = value.name.lower()
        return self.execute_insert(self.create_insert_query(GKEEP_MAP_TABLE, value))

    def get_condition_string(
        self, search_critery: Any | None = None, use_and: bool = True
    ) -> str:
        result: str = ""
        if nn(search_critery):
            condition_part_list: list[str] = []
            field_value: Any = None
            field_value_string: str | None = None
            compare_sign: str | None = None
            for field_name in search_critery.__dataclass_fields__:
                field_name: str = field_name
                field_value = search_critery.__getattribute__(field_name)
                if field_name.lower().find("id") != -1:
                    if nn(field_value) and field_value < 0:
                        return f"order by {field_name} desc limit {abs(field_value)}"
                if nn(field_value):
                    if isinstance(field_value, list):
                        field_value_string = A.D.list_to_string(
                            field_value, separator=", ", start="(", end=")"
                        )
                        compare_sign = "in"
                    else:
                        compare_sign = "="
                        field_value_string = str(field_value)
                        if field_name.lower().find(A.CT_FNC.DATE.lower()) != -1:
                            if e(field_value_string):
                                compare_sign = "is"
                                field_value_string = "NULL"
                            else:
                                field_value_string = escs(field_value_string)
                        else:
                            if isinstance(field_value, str):
                                for sign in SIGN_COLLECTION + [KEYWORD_COLLECTION.LIKE]:
                                    if field_value.startswith(sign):
                                        compare_sign = sign
                                        field_value = field_value[len(sign) :]
                                        field_value = field_value.lstrip()
                                        break
                                field_value_string = escs(field_value)
                    condition_part_list.append(
                        js((field_name, compare_sign, field_value_string))
                    )
            if len(condition_part_list) > 0:
                result = js(
                    ("where", j(condition_part_list, " and " if use_and else " or "))
                )
        return result

    def get_delayed_messages(
        self,
        search_critery: MessageSearchCritery | None = None,
        take_to_work: bool = False,
    ) -> list[DelayedMessageDS]:
        def action(connection: PooledMySQLConnection) -> list[DelayedMessageDS]:
            result: list[DelayedMessageDS] = []
            cursor = connection.cursor(dictionary=True)
            cursor.execute(
                js(
                    (
                        "select * from",
                        MESSAGE_BUFFER_TABLE_NAME,
                        self.get_condition_string(search_critery),
                    )
                )
            )

            for item in cursor:
                message: DelayedMessageDS = A.D.fill_data_from_source(
                    DelayedMessageDS(), {**item}
                )
                result.append(message)
            if take_to_work:
                status_at_work: int = A.D.get(MessageStatuses.AT_WORK)
                if ne(result):
                    id_list: list[int] = []

                    def every_action(item: DelayedMessageDS) -> None:
                        item.status = status_at_work
                        id_list.append(item.id)

                    A.D.every(every_action, result)
                    condition_statement_string = j(
                        (
                            js(("where", A.CT_FNC.ID, "in")),
                            A.D.list_to_string(id_list, start="(", end=")"),
                        )
                    )
                    cursor.execute(
                        js(
                            (
                                "update",
                                MESSAGE_BUFFER_TABLE_NAME,
                                "set",
                                A.CT_FNC.STATUS,
                                "=",
                                status_at_work,
                                condition_statement_string,
                            )
                        )
                    )
                    connection.commit()
            cursor.close()
            return result

        return self.execute(action, [])

    def update_delayed_message(
        self, value: DelayedMessageDS, search_critery: MessageSearchCritery
    ) -> bool:
        result: bool = False

        def action(connection: PooledMySQLConnection) -> bool:
            cursor = connection.cursor()
            set_statement_string: str | None = None
            field_value: Any = None
            field_value_string: str | None = None
            set_statement_part_list: list[str] = []
            for field_name in value.__dataclass_fields__:
                field_value = value.__getattribute__(field_name)
                if nn(field_value):
                    if isinstance(field_value, str):
                        field_value_string = escs(field_value)
                    else:
                        field_value_string = str(field_value)
                    set_statement_part_list.append(
                        f"{field_name} = {field_value_string}"
                    )
            if len(set_statement_part_list) > 0:
                set_statement_string = "set " + A.D.list_to_string(
                    set_statement_part_list, separator=", "
                )
                cursor.execute(
                    f"update {MESSAGE_BUFFER_TABLE_NAME} {set_statement_string} {self.get_condition_string(search_critery)}"
                )
                connection.commit()
                result = cursor.rowcount == 1
            return result

        return self.execute(action, result)

    def execute_insert(self, query: str) -> bool | None:
        def action(connection: PooledMySQLConnection) -> bool:
            cursor = connection.cursor()
            cursor.execute(query)
            connection.commit()
            return cursor.rowcount >= 1

        return self.execute(action, None)

    def add_polibase_person_visit(self, value: PolibasePersonVisitDS) -> bool:
        return self.execute_insert(
            f"replace into {POLIBASE_PERSON_VISITS_TABLE_NAME} ({A.CT_FNC.ID}, {A.CT_FNC.PIN}, {A.CT_FNC.FULL_NAME}, {A.CT_FNC.TELEPHONE_NUMBER}, {A.CT_FNC.REGISTRATION_DATE}, {A.CT_FNC.BEGIN_DATE}, {A.CT_FNC.COMPLETE_DATE}, {A.CT_FNC.STATUS}, {A.CT_FNC.DOCTOR_ID}, {A.CT_FNC.DOCTOR_FULL_NAME}, {A.CT_FNC.SERVICE_GROUP_ID}) value ({value.id}, {value.pin}, '{value.FullName}', '{value.telephoneNumber}', '{value.registrationDate}', '{value.beginDate}', '{value.completeDate}', {value.status}, {value.doctorID}, '{value.doctorFullName}', {value.serviceGroupID})"
        )

    def register_polibase_person_visit_notification(
        self, value: PolibasePersonVisitNotificationDS
    ) -> bool:
        return self.execute_insert(
            f"insert into {POLIBASE_PERSON_VISIT_NOTIFICATIONS_TABLE_NAME} ({A.CT_FNC.VISIT_ID}, {A.CT_FNC.MESSAGE_ID}, {A.CT_FNC.TYPE}) value ({value.visitID}, {value.messageID}, {value.type})"
        )

    def update_polibase_person_notification_confirmation(
        self, value: PolibasePersonNotificationConfirmation
    ) -> bool:
        return self.execute_insert(
            f"replace into {POLIBASE_PERSON_NOTIFICATION_CONFIRMATIONS_TABLE_NAME} ({A.CT_FNC.RECIPIENT}, {A.CT_FNC.SENDER}, {A.CT_FNC.STATUS}) value ('{value.recipient}', '{value.sender}', '{value.status}')"
        )

    def search_polibase_person_notification_confirmation(
        self, value: PolibasePersonNotificationConfirmation
    ) -> PolibasePersonNotificationConfirmation:
        result: PolibasePersonNotificationConfirmation | None = None

        def action(
            connection: PooledMySQLConnection,
        ) -> PolibasePersonNotificationConfirmation:
            cursor = connection.cursor(dictionary=True)
            cursor.execute(
                js(
                    (
                        "select * from",
                        POLIBASE_PERSON_NOTIFICATION_CONFIRMATIONS_TABLE_NAME,
                        "where",
                        A.CT_FNC.RECIPIENT,
                        "=",
                        escs(value.recipient),
                        "and",
                        A.CT_FNC.SENDER,
                        "=",
                        escs(value.sender),
                    )
                )
            )
            result = A.D.fill_data_from_source(
                PolibasePersonNotificationConfirmation(), cursor.fetchone()
            )
            return result

        return self.execute(action, result)

    def register_delayed_message(self, message: DelayedMessage) -> int:
        value_statement_string: str = ""
        fields_statement_string: str = ""
        value_statement_part_list: list[str] = []
        field_part_list: list[str] = []
        for field_name in message.__dataclass_fields__:
            field_value = message.__getattribute__(field_name)
            if nn(field_value):
                if isinstance(field_value, str):
                    field_value_string = A.D_F.as_string(
                        field_value, escaped_string=True
                    )
                else:
                    field_value_string = str(field_value)
                field_part_list.append(field_name)
                value_statement_part_list.append(field_value_string)
        if len(value_statement_part_list) > 0:
            value_statement_string = "value " + A.D.list_to_string(
                value_statement_part_list, start="(", end=")"
            )
            fields_statement_string = A.D.list_to_string(
                field_part_list, start="(", end=")"
            )
        if self.execute_insert(
            f"insert into {MESSAGE_BUFFER_TABLE_NAME} {fields_statement_string} {value_statement_string}"
        ):

            def action(connection: PooledMySQLConnection) -> int:
                cursor = connection.cursor()
                cursor.execute(
                    f"select max({A.CT_FNC.ID}) from {MESSAGE_BUFFER_TABLE_NAME}"
                )
                return cursor.fetchone()[0]

            return self.execute(action, 0)
        return 0



import ipih

from pih.consts.hosts import Hosts
from pih.collections.service import ServiceDescription

from enum import IntEnum, auto

NAME: str = "DataSource"

DATABASE_NAME: str = "pih_db"
DATABASE_PORT: int = 3306
DATABASE_POOL_SIZE: int = 5

HOST = Hosts.BACKUP_WORKER

VERSION: str = "0.12"

MODULES: tuple[str, ...] = ("mysql-connector-python", "lmdb==1.4.0", "lmdbm", "dadata")

SD: ServiceDescription = ServiceDescription(
    name=NAME,
    description="Data storage and source service",
    host=HOST.NAME,
    commands=(
        "register_polibase_person_information_quest",
        "search_polibase_person_information_quests",
        "update_polibase_person_information_quest",
        #
        "update_polibase_person_visit_to_data_stogare",
        "search_polibase_person_visits_in_data_storage",
        #
        "register_polibase_person_visit_notification",
        "search_polibase_person_visit_notifications",
        #
        "register_delayed_message",
        "search_delayed_messages",
        "update_delayed_message",
        #
        "get_settings_value",
        "set_settings_value",
        #
        "search_polibase_person_notification_confirmation",
        "update_polibase_person_notification_confirmation",
        #
        "get_storage_value",
        "set_storage_value",
        #
        "get_ogrn_value",
        "get_fms_unit_name",
        #
        "register_chiller_indications_value",
        "register_ct_indications_value",
        "get_last_ct_indications_value_container_list",
        "get_last_chiller_indications_value_container_list",
        #
        "get_gkeep_item_id",
        "add_gkeep_map_item",
        #
        "register_event",
        "get_event",
        "remove_event",
        #
        "execute_data_source_query"
        #
        "joke",
        "get_event_count",
    ),
    standalone_name="ds",
    version=VERSION,
    use_standalone=True,
    packages=MODULES
)


class ACTIONS(IntEnum):
    SELECT = auto()
    DELETE = auto()
    INSERT = auto()
    COUNT = auto()


ANY_SYMBOL: str = "%"
SIGN_COLLECTION: list[str] = ["!=", ">=", "<=", "<>", ">", "<"]

POLIBASE_PERSON_INFORMATION_QUESTS_TABLE_NAME: str = (
    "polibase_person_information_quests"
)
POLIBASE_PERSON_VISITS_TABLE_NAME: str = "polibase_person_visits"
POLIBASE_PERSON_VISIT_NOTIFICATIONS_TABLE_NAME: str = (
    "polibase_person_visit_notifications"
)
POLIBASE_PERSON_NOTIFICATION_CONFIRMATIONS_TABLE_NAME: str = (
    "polibase_person_notification_confirmations"
)
#
CT_INDICATIONS_VALUE_TABLE_NAME: str = "ct_indications_value"
CHILLER_INDICATIONS_VALUE_TABLE_NAME: str = "chiller_indications_value"
#
GKEEP_MAP_TABLE: str = "gkeep_map"
EVENTS_TABLE: str = "events"
MESSAGE_BUFFER_TABLE_NAME: str = "message_buffer"
#
SETTINGS_SECTION: str = "pih_settings"
STORAGE_SECTION: str = "pih_storage"


class JOKE:

    USER_PROFILE: dict[str, str] = {
        "http_method": "POST",
        "pid": "e1u5962439l73iwnci9v",
        "key": "0ba083328cd4ba19272437f2293665163c9acc47c864311a0ed1441dfd61fc48",
    }

    API_SETTINGS: dict[str, int] = {
        # "category": 2,
        "lang": 1,
        "wlist": 1,
        "markup": 0,
    }



import json
from lmdbm import Lmdb


class JsonLmdb(Lmdb):
    def _pre_key(self, value):
        return value.encode("utf-8")

    def _post_key(self, value):
        return value.decode("utf-8")

    def _pre_value(self, value):
        return json.dumps(value).encode("utf-8")

    def _post_value(self, value):
        return json.loads(value.decode("utf-8"))



import ipih

from pih import A
from DataSourceService.const import SD, JOKE

SC = A.CT_SC

ISOLATED: bool = False


def start(as_standalone: bool = False) -> None:

    if A.U.for_service(SD, as_standalone=as_standalone):

        from pih.collections import (
            OGRN,
            EventDS,
            GKeepItem,
            DelayedMessage,
            DelayedMessageDS,
            MessageSearchCritery,
            PolibasePersonVisitDS,
            CTIndicationsValueContainer,
            PolibasePersonInformationQuest,
            PolibasePersonVisitNotification,
            ChillerIndicationsValueContainer,
            PolibasePersonVisitNotificationDS,
            PolibasePersonNotificationConfirmation,
        )
        from pih.tools import ParameterList
        from pih.consts.errors import Error
        from DataSourceService.api import DataStorageAndSourceApi as Api
        from DataSourceService.anecdotica import RandomItemApi
        
        import grpc
        from typing import Any
        from dadata import Dadata
    

        api: Api = Api()

        def service_call_handler(sc: SC, pl: ParameterList, context) -> Any:
            if sc == SC.register_polibase_person_information_quest:
                return api.register_polibase_person_information_quest(
                    pl.next(PolibasePersonInformationQuest())
                )
            if sc == SC.search_polibase_person_information_quests:
                return A.R.pack(
                    None,
                    api.get_polibase_person_information_quests(
                        pl.next(PolibasePersonInformationQuest())
                    ),
                )
            if sc == SC.update_polibase_person_information_quest:
                return api.update_polibase_person_information_quest(
                    pl.next(PolibasePersonInformationQuest()),
                    pl.next(PolibasePersonInformationQuest()),
                )
            if sc == SC.get_settings_value:
                return A.R.pack(
                    A.CT_FCA.VALUE,
                    api.get_settings_value(pl.next(), pl.next()),
                )
            if sc == SC.set_settings_value:
                return api.set_settings_value(pl.next(), pl.next())
            if sc == SC.register_polibase_person_visit_notification:
                return api.register_polibase_person_visit_notification(
                    pl.next(PolibasePersonVisitNotificationDS())
                )
            if sc == SC.search_polibase_person_visit_notifications:
                return A.R.pack(
                    None,
                    api.search_polibase_person_visit_notifications(
                        pl.next(PolibasePersonVisitNotification())
                    ),
                )
            if sc == SC.update_polibase_person_visit_to_data_stogare:
                return api.add_polibase_person_visit(pl.next(PolibasePersonVisitDS()))
            if sc == SC.search_polibase_person_visits_in_data_storage:
                return A.R.pack(
                    None,
                    api.search_polibase_person_visit(pl.next(PolibasePersonVisitDS())),
                )
            if sc == SC.register_delayed_message:
                return api.register_delayed_message(pl.next(DelayedMessage()))
            if sc == SC.search_delayed_messages:
                return A.R.pack(
                    None,
                    api.get_delayed_messages(
                        pl.next(MessageSearchCritery()), pl.next()
                    ),
                )
            if sc == SC.update_delayed_message:
                return api.update_delayed_message(
                    pl.next(DelayedMessageDS()),
                    pl.next(MessageSearchCritery()),
                )
            if sc == SC.get_storage_value:
                return A.R.pack(
                    A.CT_FCA.VALUE,
                    api.get_storage_value(pl.next(), pl.next()),
                )
            if sc == SC.set_storage_value:
                return api.set_storage_value(pl.next(), pl.next(), pl.next())
            if sc == SC.update_polibase_person_notification_confirmation:
                return api.update_polibase_person_notification_confirmation(
                    pl.next(PolibasePersonNotificationConfirmation())
                )
            if sc == SC.search_polibase_person_notification_confirmation:
                return A.R.pack(
                    None,
                    api.search_polibase_person_notification_confirmation(
                        pl.next(PolibasePersonNotificationConfirmation())
                    ),
                )
            if sc == SC.register_ct_indications_value:
                return api.register_ct_indication_value(
                    pl.next(CTIndicationsValueContainer(A.D.now(second=0))),
                    pl.next(),
                )
            if sc == SC.register_chiller_indications_value:
                return api.register_chiller_indication_value(
                    pl.next(ChillerIndicationsValueContainer(A.D.now(second=0))),
                    pl.next(),
                )
            if sc == SC.execute_data_source_query:
                try:
                    return A.R.pack(A.CT_FCA.VALUE, api.execute_query(pl.next()))
                except Error as error:
                    return A.ER.rpc(
                        context, error.details, grpc.StatusCode.INVALID_ARGUMENT
                    )
            if sc == SC.get_ogrn_value:
                section_name: str = "OGRN"
                ogrn: OGRN | None = None
                ogrn_code: str = str(pl.next())
                data: dict | None = api.get_storage_value(ogrn_code, section_name)
                if A.D.is_empty(data):
                    token: str = A.D_V_E.value("DADATA_TOKEN")
                    dadata: Dadata = Dadata(token)
                    result: list[dict] = dadata.find_by_id("party", ogrn_code)
                    if not A.D_C.empty(result):
                        data = result[0]
                        ogrn = OGRN(data["value"], ogrn_code, data)
                        api.set_storage_value(ogrn_code, data, section_name)
                else:
                    ogrn = OGRN(data["value"], ogrn_code, data)
                return A.R.pack(A.CT_FCA.VALUE, ogrn)
            if sc == SC.get_fms_unit_name:
                section_name: str = "FMS_UNIT"
                fms_unit_name: str | None = None
                fms_unit_code: str = str(pl.next())
                data: str | None = api.get_storage_value(fms_unit_code, section_name)
                if A.D.is_empty(data):
                    token: str = A.D_V_E.value("DADATA_TOKEN")
                    dadata: Dadata = Dadata(token)
                    result: list[dict] = dadata.find_by_id("fms_unit", fms_unit_code)
                    if not A.D_C.empty(result):
                        data = result[0]
                        fms_unit_name = data["value"]
                        api.set_storage_value(
                            fms_unit_code, fms_unit_name, section_name
                        )
                else:
                    fms_unit_name = data
                return A.R.pack(A.CT_FCA.VALUE, fms_unit_name)
            if sc == SC.get_last_ct_indications_value_container_list:
                return A.R.pack(
                    A.CT_FC.INDICATIONS.CT_VALUE_CONTAINER,
                    api.get_last_ct_indications_value_container_list(
                        pl.next(), pl.next()
                    ),
                )
            if sc == SC.get_last_сhiller_indications_value_container_list:
                return A.R.pack(
                    A.CT_FC.INDICATIONS.CHILLER_VALUE_CONTAINER,
                    api.get_last_chiller_indications_value_container_list(
                        pl.next(), pl.next()
                    ),
                )
            if sc == SC.get_gkeep_item_list_by_any:
                return A.R.pack(
                    None,
                    api.get_gkeep_item_list_by_any(pl.next(), pl.next(), pl.next()),
                )
            if sc == SC.add_gkeep_item:
                return A.R.pack(
                    A.CT_FC.VALUE,
                    api.add_gkeep_item(
                        GKeepItem(
                            pl.next(),
                            pl.next(),
                            pl.next(),
                        )
                    ),
                )
            if sc == SC.register_event:
                return api.register_event(pl.next(EventDS()))
            if sc == SC.get_event:
                return A.R.pack(A.CT_FC.VALUE, api.get_event(pl.next(EventDS())))
            if sc == SC.get_event_count:
                return A.R.pack(A.CT_FC.VALUE, api.get_event_count(pl.next(EventDS())))
            if sc == SC.remove_event:
                return api.remove_event(pl.next(EventDS()))
            if sc == SC.joke:
                reply = RandomItemApi.get_reply(
                    JOKE.USER_PROFILE, JOKE.API_SETTINGS
                )
                if reply.is_error():
                    print(
                        reply.get_result().get_error(), reply.get_result().get_err_msg()
                    )
                else:
                    return A.R.pack(A.CT_FC.VALUE, reply.get_item().get_text())

        A.SRV_A.serve(
            SD, service_call_handler, isolate=ISOLATED, as_standalone=as_standalone
        )


if __name__ == "__main__":
    start()
